// ToolbarDoc.cpp : implementation of the CToolbarDoc class
//

#include "stdafx.h"
#include "Toolbar.h"
#include "MainFrm.h"
#include "Line.h"
#include "ToolbarDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CToolbarDoc

IMPLEMENT_DYNCREATE(CToolbarDoc, CDocument)

BEGIN_MESSAGE_MAP(CToolbarDoc, CDocument)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COLOR, OnUpdateIndicatorColor)
	//{{AFX_MSG_MAP(CToolbarDoc)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_VTHIN, OnUpdateWidthVthin)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_VTHICK, OnUpdateWidthVthick)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_THIN, OnUpdateWidthThin)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_THICK, OnUpdateWidthThick)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_MEDIUM, OnUpdateWidthMedium)
	ON_UPDATE_COMMAND_UI(ID_COLOR_YELLOW, OnUpdateColorYellow)
	ON_UPDATE_COMMAND_UI(ID_COLOR_WHITE, OnUpdateColorWhite)
	ON_UPDATE_COMMAND_UI(ID_COLOR_RED, OnUpdateColorRed)
	ON_UPDATE_COMMAND_UI(ID_COLOR_MAGENTA, OnUpdateColorMagenta)
	ON_UPDATE_COMMAND_UI(ID_COLOR_GREEN, OnUpdateColorGreen)
	ON_UPDATE_COMMAND_UI(ID_COLOR_CYAN, OnUpdateColorCyan)
	ON_UPDATE_COMMAND_UI(ID_COLOR_BLUE, OnUpdateColorBlue)
	ON_UPDATE_COMMAND_UI(ID_COLOR_BLACK, OnUpdateColorBlack)
	ON_COMMAND(ID_COLOR_BLACK, OnColorBlack)
	ON_COMMAND(ID_COLOR_BLUE, OnColorBlue)
	ON_COMMAND(ID_COLOR_CYAN, OnColorCyan)
	ON_COMMAND(ID_COLOR_GREEN, OnColorGreen)
	ON_COMMAND(ID_COLOR_MAGENTA, OnColorMagenta)
	ON_COMMAND(ID_COLOR_RED, OnColorRed)
	ON_COMMAND(ID_COLOR_WHITE, OnColorWhite)
	ON_COMMAND(ID_COLOR_YELLOW, OnColorYellow)
	ON_COMMAND(ID_WIDTH_MEDIUM, OnWidthMedium)
	ON_COMMAND(ID_WIDTH_THICK, OnWidthThick)
	ON_COMMAND(ID_WIDTH_THIN, OnWidthThin)
	ON_COMMAND(ID_WIDTH_VTHICK, OnWidthVthick)
	ON_COMMAND(ID_WIDTH_VTHIN, OnWidthVthin)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

const COLORREF CToolbarDoc::m_crColors[8] = {
	RGB(   0,   0,   0),	// Black
	RGB(   0,   0, 255),	// Blue
	RGB(   0, 255,   0),	// Green
	RGB(   0, 255, 255),	// Cyan
	RGB( 255,   0,   0),	// Red
	RGB( 255,   0, 255),	// Magenta
	RGB( 255, 255,   0),	// Yellow
	RGB( 255, 255, 255)		// White
};

/////////////////////////////////////////////////////////////////////////////
// CToolbarDoc construction/destruction

CToolbarDoc::CToolbarDoc()
{
	// TODO: add one-time construction code here

}

CToolbarDoc::~CToolbarDoc()
{
}

BOOL CToolbarDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	///////////////////////
	// MY CODE STARTS HERE
	///////////////////////

	// Initialize the color to black
	m_nColor = ID_COLOR_BLACK - ID_COLOR_BLACK;
	m_nWidth = ID_WIDTH_MEDIUM - ID_WIDTH_VTHIN;
	UpdateColorbar(m_nWidth);

	///////////////////////
	// MY CODE ENDS HERE
	///////////////////////

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CToolbarDoc serialization

void CToolbarDoc::Serialize(CArchive& ar)
{
	// Pass the serialization on to the object array
	m_oaLines.Serialize(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CToolbarDoc diagnostics

#ifdef _DEBUG
void CToolbarDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CToolbarDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CToolbarDoc commands

int CToolbarDoc::GetLineCount()
{
	// Return the array count
	return m_oaLines.GetSize();
}

CPoint* CToolbarDoc::GetTo(int nIndex)
{
	// Return a pointer to the To point in the
	// CLine object at the specified point in the
	// object array
	return ((CLine*)m_oaLines[nIndex])->GetTo();
}

CPoint* CToolbarDoc::GetFrom(int nIndex)
{
	// Return a pointer to the From point in the
	// CLine object at the specified point in the
	// object array
	return ((CLine*)m_oaLines[nIndex])->GetFrom();
}

CLine * CToolbarDoc::AddLine(CPoint ptFrom, CPoint ptTo)
{
	static UINT nWidths[5] = { 1, 8, 16, 24, 32};

	// Create a new CLine object
	CLine *pLine = new CLine(ptFrom, ptTo, nWidths[m_nWidth], m_crColors[m_nColor]);
	try
	{
		// Add the new line to the object array
		m_oaLines.Add(pLine);
		// Mark the document as dirty
		SetModifiedFlag();
	}
	// Did we run into a memory exception?
	catch (CMemoryException* perr)
	{
		// Display a message for the user, giving them the
		// bad news
		AfxMessageBox("Out of memory", MB_ICONSTOP | MB_OK);
		// Did we create a line object?
		if (pLine)
		{
			// Delete it
			delete pLine;
			pLine = NULL;
		}
		// Delete the exception object
		perr->Delete();
	}
	return pLine;
}

void CToolbarDoc::DeleteContents() 
{
	// TODO: Add your specialized code here and/or call the base class
	
	///////////////////////
	// MY CODE STARTS HERE
	///////////////////////

	// Get the number of lines in the object array
	int liCount = m_oaLines.GetSize();
	int liPos;

	// Are there any objects in the array?
	if (liCount)
	{
		// Loop through the array, deleting each object
		for (liPos = 0; liPos < liCount; liPos++)
			delete m_oaLines[liPos];
		// Reset the array
		m_oaLines.RemoveAll();
	}

	///////////////////////
	// MY CODE ENDS HERE
	///////////////////////

	CDocument::DeleteContents();
}

CLine * CToolbarDoc::GetLine(int nIndex)
{
	// Return a pointer to the line object
	// at the specified point in the object array
	return (CLine*)m_oaLines[nIndex];
}

UINT CToolbarDoc::GetColor()
{
	// Return the current color
	return ID_COLOR_BLACK + m_nColor;
}

UINT CToolbarDoc::GetWidth()
{
	// Return the current width
	return ID_WIDTH_VTHIN + m_nWidth;
}

void CToolbarDoc::OnUpdateWidthVthin(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Very Thin menu entry should be checked
//	pCmdUI->SetCheck(GetWidth() == ID_WIDTH_VTHIN ? 1 : 0);
	pCmdUI->SetRadio(GetWidth() == ID_WIDTH_VTHIN ? 1 : 0);
}

void CToolbarDoc::OnUpdateWidthVthick(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Very Thick menu entry should be checked
//	pCmdUI->SetCheck(GetWidth() == ID_WIDTH_VTHICK ? 1 : 0);
	pCmdUI->SetRadio(GetWidth() == ID_WIDTH_VTHICK ? 1 : 0);
}

void CToolbarDoc::OnUpdateWidthThin(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Thin menu entry should be checked
//	pCmdUI->SetCheck(GetWidth() == ID_WIDTH_THIN ? 1 : 0);
	pCmdUI->SetRadio(GetWidth() == ID_WIDTH_THIN ? 1 : 0);
}

void CToolbarDoc::OnUpdateWidthThick(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Thick menu entry should be checked
//	pCmdUI->SetCheck(GetWidth() == ID_WIDTH_THICK ? 1 : 0);
	pCmdUI->SetRadio(GetWidth() == ID_WIDTH_THICK ? 1 : 0);
}

void CToolbarDoc::OnUpdateWidthMedium(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Medium menu entry should be checked
//	pCmdUI->SetCheck(GetWidth() == ID_WIDTH_MEDIUM ? 1 : 0);
	pCmdUI->SetRadio(GetWidth() == ID_WIDTH_MEDIUM ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorYellow(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Yellow menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_YELLOW ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_YELLOW ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorWhite(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the White menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_WHITE ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_WHITE ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorRed(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Red menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_RED ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_RED ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorMagenta(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Magenta menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_MAGENTA ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_MAGENTA ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorGreen(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Green menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_GREEN ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_GREEN ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorCyan(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Cyan menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_CYAN ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_CYAN ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorBlue(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Blue menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_BLUE ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_BLUE ? 1 : 0);
}

void CToolbarDoc::OnUpdateColorBlack(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	// Determine if the Black menu entry should be checked
//	pCmdUI->SetCheck(GetColor() == ID_COLOR_BLACK ? 1 : 0);
	pCmdUI->SetRadio(GetColor() == ID_COLOR_BLACK ? 1 : 0);
}

void CToolbarDoc::OnColorBlack() 
{
	// TODO: Add your command handler code here

	// Set the current color to black
	m_nColor = 0;	
}

void CToolbarDoc::OnColorBlue() 
{
	// TODO: Add your command handler code here

	// Set the current color to blue
	m_nColor = 1;
}

void CToolbarDoc::OnColorCyan() 
{
	// TODO: Add your command handler code here

	// Set the current color to cyan
	m_nColor = 3;
}

void CToolbarDoc::OnColorGreen() 
{
	// TODO: Add your command handler code here

	// Set the current color to green
	m_nColor = 2;
}

void CToolbarDoc::OnColorMagenta() 
{
	// TODO: Add your command handler code here

	// Set the current color to magenta
	m_nColor = 5;
}

void CToolbarDoc::OnColorRed() 
{
	// TODO: Add your command handler code here

	// Set the current color to red
	m_nColor = 4;
}

void CToolbarDoc::OnColorWhite() 
{
	// TODO: Add your command handler code here

	// Set the current color to white
	m_nColor = 7;
}

void CToolbarDoc::OnColorYellow() 
{
	// TODO: Add your command handler code here

	// Set the current color to yellow
	m_nColor = 6;
}

void CToolbarDoc::OnWidthMedium() 
{
	// TODO: Add your command handler code here

	// Set the current width to medium
	m_nWidth = 2;
	// Update the combo box on the color toolbar
	UpdateColorbar(2);
}

void CToolbarDoc::OnWidthThick() 
{
	// TODO: Add your command handler code here

	// Set the current width to thick
	m_nWidth = 3;
	// Update the combo box on the color toolbar
	UpdateColorbar(3);
}

void CToolbarDoc::OnWidthThin() 
{
	// TODO: Add your command handler code here

	// Set the current width to thin
	m_nWidth = 1;
	// Update the combo box on the color toolbar
	UpdateColorbar(1);
}

void CToolbarDoc::OnWidthVthick() 
{
	// TODO: Add your command handler code here

	// Set the current width to very thick
	m_nWidth = 4;
	// Update the combo box on the color toolbar
	UpdateColorbar(4);
}

void CToolbarDoc::OnWidthVthin() 
{
	// TODO: Add your command handler code here

	// Set the current width to very thin
	m_nWidth = 0;
	// Update the combo box on the color toolbar
	UpdateColorbar(0);
}

void CToolbarDoc::SetWidth(int nIndex)
{
	m_nWidth = nIndex;
}


void CToolbarDoc::UpdateColorbar(int nIndex)
{
	// Get the position of the first view
	POSITION pos = GetFirstViewPosition();
	// Did we get a valid position?
	if (pos != NULL)
	{
		// Get a pointer to the view in that position
		CView* pView = GetNextView(pos);
		// Do we have a valid pointer to the view?
		if (pView)
		{
			// Get a pointer to the frame through the view
			CMainFrame* pFrame = (CMainFrame*)pView->GetTopLevelFrame();
			// Did we get a pointer to the frame?
			if (pFrame)
				// Update the combo box on the color toolbar 
				// through the frame
				pFrame->UpdateWidthCB(nIndex);
		}
	}
}

void CToolbarDoc::OnUpdateIndicatorColor(CCmdUI *pCmdUI)
{
	CString strColor;

	// What is the current color?
	switch (m_nColor)
	{
	case 0:	// Black
		strColor = "BLACK";
		break;
	case 1:	// Blue
		strColor = "BLUE";
		break;
	case 2:	// Green
		strColor = "GREEN";
		break;
	case 3:	// Cyan
		strColor = "CYAN";
		break;
	case 4:	// Red
		strColor = "RED";
		break;
	case 5:	// Magenta
		strColor = "MAGENTA";
		break;
	case 6:	// Yellow
		strColor = "YELLOW";
		break;
	case 7:	// White
		strColor = "WHITE";
		break;
	}
	// Enable the status bar pane
	pCmdUI->Enable(TRUE);
	// Set the text of the status bar pane
	// to the current color
	pCmdUI->SetText(strColor);
}
