// DbAdoDoc.cpp : implementation of the CDbAdoDoc class
//

#include "stdafx.h"
#include "DbAdo.h"

#include "CustomRS.h"
#include "DbAdoDoc.h"
#include "DbAdoView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDbAdoDoc

IMPLEMENT_DYNCREATE(CDbAdoDoc, CDocument)

BEGIN_MESSAGE_MAP(CDbAdoDoc, CDocument)
	//{{AFX_MSG_MAP(CDbAdoDoc)
	ON_UPDATE_COMMAND_UI(IDM_DATA_FIRST, OnUpdateDataFirst)
	ON_UPDATE_COMMAND_UI(IDM_DATA_LAST, OnUpdateDataLast)
	ON_UPDATE_COMMAND_UI(IDM_DATA_NEXT, OnUpdateDataNext)
	ON_UPDATE_COMMAND_UI(IDM_DATA_PREV, OnUpdateDataPrev)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDbAdoDoc construction/destruction

CDbAdoDoc::CDbAdoDoc()
{
	// TODO: add one-time construction code here
	m_piAdoRecordBinding = NULL;
	m_pRs = NULL;

}

CDbAdoDoc::~CDbAdoDoc()
{
}

BOOL CDbAdoDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	// Set the connection and SQL command strings
	m_strConnection = _T("Provider=MSDASQL.1;Data Source=TYVCDB");
	m_strCmdText = _T("select * from Addresses");

	// Initialize the Recordset and binding pointers
	m_pRs = NULL;
	m_piAdoRecordBinding = NULL;
	// Initialize the COM environment
	::CoInitialize(NULL);
	try
	{
		// Create the record set object
		m_pRs.CreateInstance(__uuidof(Recordset));

		// Open the record set object
		m_pRs->Open((LPCTSTR)m_strCmdText, (LPCTSTR)m_strConnection, adOpenDynamic,
			adLockOptimistic, adCmdUnknown);

		// Get a pointer to the record binding interface
		if (FAILED(m_pRs->QueryInterface(__uuidof(IADORecordBinding), (LPVOID *)&m_piAdoRecordBinding)))
			_com_issue_error(E_NOINTERFACE);
		// Bind the record class to the record set
		m_piAdoRecordBinding->BindToRecordset(&m_rsRecSet);

		// Get a pointer to the view
		POSITION pos = GetFirstViewPosition();
		CDbAdoView* pView = (CDbAdoView*)GetNextView(pos);
		if (pView)
			// Sync the data set with the form
			pView->RefreshBoundData();
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Display the error
		GenerateError(e.Error(), e.Description());
	}

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CDbAdoDoc serialization

void CDbAdoDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDbAdoDoc diagnostics

#ifdef _DEBUG
void CDbAdoDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDbAdoDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDbAdoDoc commands

void CDbAdoDoc::MoveFirst()
{
	try
	{
		// Update the current record
		m_piAdoRecordBinding->Update(&m_rsRecSet);
		// Move to the first record
		m_pRs->MoveFirst();
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Generate the error message
		GenerateError(e.Error(), e.Description());
	}
}

void CDbAdoDoc::MovePrev()
{
	try
	{
		// Update the current record
		m_piAdoRecordBinding->Update(&m_rsRecSet);
		// Move to the previous record
		m_pRs->MovePrevious();
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Generate the error message
		GenerateError(e.Error(), e.Description());
	}
}

void CDbAdoDoc::MoveNext()
{
	try
	{
		// Update the current record
		m_piAdoRecordBinding->Update(&m_rsRecSet);
		// Move to the next record
		m_pRs->MoveNext();
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Generate the error message
		GenerateError(e.Error(), e.Description());
	}
}

void CDbAdoDoc::MoveLast()
{
	try
	{
		// Update the current record
		m_piAdoRecordBinding->Update(&m_rsRecSet);
		// Move to the last record
		m_pRs->MoveLast();
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Generate the error message
		GenerateError(e.Error(), e.Description());
	}
}

CCustomRs* CDbAdoDoc::GetRecSet()
{
	// Return a pointer to the record object
	return &m_rsRecSet;
}

void CDbAdoDoc::DeleteContents() 
{
	// TODO: Add your specialized code here and/or call the base class
	// Close the record set
	if (m_pRs)
		m_pRs->Close();
	// Do we have a valid pointer to the record binding?
	if (m_piAdoRecordBinding)
		// Release it
		m_piAdoRecordBinding->Release();
	// Set the record set pointer to NULL
	m_pRs = NULL;

	// Shut down the COM environment
	CoUninitialize();

	CDocument::DeleteContents();
}

void CDbAdoDoc::GenerateError(HRESULT hr, PWSTR pwszDescription)
{
	CString strError;

	// Format and display the error message
	strError.Format("Run-time error '%d (%x)'", hr, hr);
	strError += "\n\n";
	strError += pwszDescription;

	AfxMessageBox(strError);
}

void CDbAdoDoc::AddNew()
{
	try
	{
		// Update the current record
		m_piAdoRecordBinding->Update(&m_rsRecSet);
		// Can we add a new record?
		if (m_pRs->Supports(adAddNew))
		{
			// Create a blank record
			CreateBlankRecord();
			// Add the blank record
			m_piAdoRecordBinding->AddNew(&m_rsRecSet);
			// Move to the last record
			m_pRs->MoveLast();
		}
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Generate an error message
		GenerateError(e.Error(), e.Description());
	}
}

void CDbAdoDoc::Delete()
{
	try
	{
		// Update the current record
		m_piAdoRecordBinding->Update(&m_rsRecSet);
		// Can we delete a record?
		if (m_pRs->Supports(adDelete))
		{
			// Make sure the user wants to delete this record
			if (AfxMessageBox("Are you sure you want to delete this record?",
					//"Delete this record?", MB_YESNO | MB_ICONQUESTION) == IDYES)
					MB_YESNO | MB_ICONQUESTION) == IDYES)
			{
				// Delete the record
				m_pRs->Delete(adAffectCurrent);
				// Move to the previous record
				m_pRs->MovePrevious();
			}
		}
	}
	// Any errors?
	catch (_com_error &e)
	{
		// Generate an error message
		GenerateError(e.Error(), e.Description());
	}
}

void CDbAdoDoc::CreateBlankRecord()
{
	// Create the blank values to be used
	CString strBlank = " ";
	COleDateTime dtBlank;

	// Set each of the values in the record object
	m_rsRecSet.m_lAddressID = 0;
	strcpy(m_rsRecSet.m_szFirstName, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szLastName, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szSpouseName, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szAddress, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szCity, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szStateOrProvince, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szPostalCode, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szCountry, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szEmailAddress, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szHomePhone, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szWorkPhone, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szWorkExtension, (LPCTSTR)strBlank);
	strcpy(m_rsRecSet.m_szFaxNumber, (LPCTSTR)strBlank);
	m_rsRecSet.m_dtBirthdate = (DATE)dtBlank;
	m_rsRecSet.m_bSendCard = VARIANT_FALSE;
	strcpy(m_rsRecSet.m_szNotes, (LPCTSTR)strBlank);
}

void CDbAdoDoc::OnUpdateDataFirst(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	// Does the record set exist?
	if (m_pRs)
	{
		// Are we at the BOF?
		if (m_pRs->BOF)
			pCmdUI->Enable(FALSE);
		else
			pCmdUI->Enable(TRUE);
	}
}

void CDbAdoDoc::OnUpdateDataLast(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	// Does the record set exist?
	if (m_pRs)
	{
		// Are we at the EOF?
		if (m_pRs->EndOfFile)
			pCmdUI->Enable(FALSE);
		else
			pCmdUI->Enable(TRUE);
	}
}

void CDbAdoDoc::OnUpdateDataNext(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	// Does the record set exist?
	if (m_pRs)
	{
		// Are we at the EOF?
		if (m_pRs->EndOfFile)
			pCmdUI->Enable(FALSE);
		else
			pCmdUI->Enable(TRUE);
	}
}

void CDbAdoDoc::OnUpdateDataPrev(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	// Does the record set exist?
	if (m_pRs)
	{
		// Are we at the BOF?
		if (m_pRs->BOF)
			pCmdUI->Enable(FALSE);
		else
			pCmdUI->Enable(TRUE);
	}
}
