// ModArt.cpp: implementation of the CModArt class.
//
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <time.h>

#include "stdafx.h"
#include "Line.h"
#include "ModArt.h"

const COLORREF CModArt::m_crColors[8] = {
	RGB(   0,   0,   0),	// Black
	RGB(   0,   0, 255),	// Blue
	RGB(   0, 255,   0),	// Green
	RGB(   0, 255, 255),	// Cyan
	RGB( 255,   0,   0),	// Red
	RGB( 255,   0, 255),	// Magenta
	RGB( 255, 255,   0),	// Yellow
	RGB( 255, 255, 255)		// White
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CModArt::CModArt()
{
	// Initialize the random number generator
	srand((unsigned)time(NULL));
	// Initialize the property variables
	m_iLength = 200;
	m_iSegments = 50;
}

CModArt::~CModArt()
{

}

void CModArt::SetRect(CRect rDrawArea)
{
	// Set the drawing area rectangle
	m_rDrawArea = rDrawArea;
}

void CModArt::ClearDrawing()
{
	// Get the number of lines in the object array
	int liCount = m_oaLines.GetSize();
	int liPos;

	// Are there any objects in the array?
	if (liCount)
	{
		// Loop through the array, deleting each object
		for (liPos = 0; liPos < liCount; liPos++)
			delete m_oaLines[liPos];
		// Reset the array
		m_oaLines.RemoveAll();
	}
}

void CModArt::Draw(CDC *pDC)
{
	// Get the number of lines in the object array
	int liCount = m_oaLines.GetSize();
	int liPos;

	// Are there any objects in the array?
	if (liCount)
	{
		// Loop through the array, deleting each object
		for (liPos = 0; liPos < liCount; liPos++)
			((CLine*)m_oaLines[liPos])->Draw(pDC);
	}
}

void CModArt::Serialize(CArchive &ar)
{
	// Pass the archive object on to the array
	m_oaLines.Serialize(ar);
}

void CModArt::NewDrawing()
{
	int lNumLines;
	int lCurLine;

	// Determine how many lines to create
	lNumLines = rand() % m_iSegments;
	// Are there any lines to create?
	if (lNumLines > 0)
	{
		// Loop through the number of lines
		for (lCurLine = 0; lCurLine < lNumLines; lCurLine++)
		{
			// Create the new line
			NewLine();
		}
	}
}

void CModArt::NewLine()
{
	int lNumLines;
	int lCurLine;
	int nCurColor;
	UINT nCurWidth;
	CPoint pTo;
	CPoint pFrom;
	int cRed;
	int cBlue;
	int cGreen;

	// Normalize the rectangle before determining the width and height
	m_rDrawArea.NormalizeRect();
	// get the area width and height
	int lWidth = m_rDrawArea.Width();
	int lHeight = m_rDrawArea.Height();

	// Determine the number of parts to this squiggle
	lNumLines = rand() % m_iLength;
	// Are there any parts to this squiggle?
	if (lNumLines > 0)
	{
		// Determine the color
		nCurColor = rand() % 8;
		cRed = rand() % 255;
		cBlue = rand() % 255;
		cGreen = rand() % 255;
		// Determine the pen width
		nCurWidth = (rand() % 8) + 1;
		// Determine the starting point for the squiggle
		pFrom.x = (rand() % lWidth) + m_rDrawArea.left;
		pFrom.y = (rand() % lHeight) + m_rDrawArea.top;
		// Loop through the number of segments
		for (lCurLine = 0; lCurLine < lNumLines; lCurLine++)
		{
			// Determine the end point of the segment
			pTo.x = ((rand() % 20) - 10) + pFrom.x;
			pTo.y = ((rand() % 20) - 10) + pFrom.y;
			// Create a new CLine object
//			CLine *pLine = new CLine(pFrom, pTo, nCurWidth, m_crColors[nCurColor]);
			CLine *pLine = new CLine(pFrom, pTo, nCurWidth, RGB(cRed, cGreen, cBlue));
			try
			{
				// Add the new line to the object array
				m_oaLines.Add(pLine);
			}
			// Did we run into a memory exception?
			catch (CMemoryException* perr)
			{
				// Display a message for the user, giving him the
				// bad news
				CString strMsg = "Out of memory";
				AfxMessageBox(strMsg, MB_ICONSTOP | MB_OK);
				// Did we create a line object?
				if (pLine)
				{
					// Delete it
					delete pLine;
					pLine = NULL;
				}
				// Delete the exception object
				perr->Delete();
			}
			// Set the starting point to the end point
			pFrom = pTo;
		}
	}
}

void CModArt::SetLength(int iLength)
{
	// Set the current value for the m_iLength variable
	m_iLength = iLength;
}

void CModArt::SetSegments(int iSegments)
{
	// Set the current value for the m_iSegments variable
	m_iSegments = iSegments;
}



int CModArt::GetLength()
{
	// Return the current value for the m_iLength variable
	return m_iLength;
}

int CModArt::GetSegments()
{
	// Return the current value for the m_iSegments variable
	return m_iSegments;
}
